package net.knarcraft.blockhunt.util;

import net.knarcraft.blockhunt.BlockHunt;
import net.knarcraft.blockhunt.config.ChatMessageTypeIdentifier;
import net.knarcraft.blockhunt.config.Key;
import net.md_5.bungee.api.ChatColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.util.Collection;
import java.util.Objects;
import java.util.logging.Level;

/**
 * A helper class for sending messages
 */
public final class MessageHelper {

    private MessageHelper() {

    }

    /**
     * Sends a message to a player
     *
     * <p>Also replaces the "%player%" variable to the player's name</p>
     *
     * @param player    <p>The player to receive the message</p>
     * @param message   <p>The message to send to a player</p>
     * @param variables <p>Variables specified with the format "name-value" corresponding to %name%</p>
     */
    public static void sendMessage(Player player, String message, String... variables) {
        if (message == null) {
            return;
        }
        String playerString = player == null ? "Console" : player.getDisplayName();
        message = MessageHelper.replaceAll(message.replaceAll("%player%", playerString), variables);
        Objects.requireNonNullElseGet(player, Bukkit::getConsoleSender).sendMessage(message);
    }

    /**
     * Sends a message to a player defined
     *
     * <p>Also replaces the "%player%" variable to the player's name</p>
     *
     * @param player     <p>The player to receive the message</p>
     * @param messageKey <p>A key pointing to a message</p>
     * @param variables  <p>Variables specified with the format "name-value" corresponding to %name%</p>
     */
    public static void sendMessage(Player player, Key messageKey, String... variables) {
        sendMessage(player, getMessageFromConfig(player, messageKey), variables);
    }

    /**
     * Sends a message to the specified players
     *
     * <p>Also replaces the "%player%" variable to the player's name.</p>
     *
     * @param players   <p>The players to broadcast the message to</p>
     * @param message   <p>The message to send to a player</p>
     * @param variables <p>Variables specified with the format "name-value" corresponding to %name%</p>
     */
    public static void broadcastMessage(Collection<? extends Player> players, String message, String... variables) {
        if (message == null) {
            return;
        }
        for (Player player : players) {
            sendMessage(player, message, variables);
        }
    }

    /**
     * Sends a message to all players online
     *
     * <p>Also replaces the "%player%" variable to the player's name.</p>
     *
     * @param message   <p>The message to send to a player</p>
     * @param variables <p>Variables specified with the format "name-value" corresponding to %name%</p>
     */
    public static void broadcastMessage(String message, String... variables) {
        broadcastMessage(Bukkit.getOnlinePlayers(), message, variables);
    }

    /**
     * Sends a message to the specified players
     *
     * <p>Also replaces the "%player%" variable to the player's name.</p>
     *
     * @param players    <p>The players to broadcast the message to</p>
     * @param messageKey <p>A key pointing to a message</p>
     * @param variables  <p>Variables specified with the format "name-value" corresponding to %name%</p>
     */
    public static void broadcastMessage(Collection<? extends Player> players, Key messageKey, String... variables) {
        broadcastMessage(players, getMessageFromConfig(null, messageKey), variables);
    }

    /**
     * Sends a message to all players online
     *
     * <p>Also replaces the "%player%" variable to the player's name.</p>
     *
     * @param messageKey <p>A key pointing to a message</p>
     * @param variables  <p>Variables specified with the format "name-value" corresponding to %name%</p>
     */
    public static void broadcastMessage(Key messageKey, String... variables) {
        broadcastMessage(getMessageFromConfig(null, messageKey), variables);
    }

    /**
     * Replaces all variables and color variables in a string
     *
     * @param message   <p>The message to replace variables for</p>
     * @param variables <p>Variables specified with the format "name-value" corresponding to %name%</p>
     * @return <p>The input string with any variables and color variables replaced</p>
     */
    public static String replaceAll(String message, String... variables) {
        return MessageHelper.replaceColors(MessageHelper.replaceFormattingVariables(
                MessageHelper.replaceVariables(message, variables)));
    }

    /**
     * Replaces default Minecraft colour codes
     *
     * @param message <p>The message to replace color codes for</p>
     * @return <p>The input string with any default colors replaced</p>
     */
    public static String replaceColors(String message) {
        return ChatColor.translateAlternateColorCodes('&', message);
    }

    /**
     * Replaces special formatting variables defined by this plugin
     *
     * @param message <p>The message to replace formatting placeholders for</p>
     * @return <p>The input string with any formatting placeholders replaced</p>
     */
    public static String replaceFormattingVariables(String message) {
        return ChatMessageTypeIdentifier.replaceAll(message);
    }

    /**
     * Replaces custom variables
     *
     * @param message   <p>The message to replace variables for</p>
     * @param variables <p>Variables specified with the format "name-value" corresponding to %name%</p>
     * @return <p>The input string with any variables replaced</p>
     */
    public static String replaceVariables(String message, String... variables) {
        for (String variable : variables) {
            // Note: The splitting needs to be limited, or negative numbers will become empty strings
            String[] split = variable.split("-", 2);
            message = message.replaceAll("%" + split[0] + "%", split[1]);
        }
        return message;
    }

    /**
     * Gets a message from the messages configuration file
     *
     * @param player     <p>The player to get the message for, or null</p>
     * @param messageKey <p>The key specifying the message</p>
     * @return <p>The stored message, or null if not found</p>
     */
    private static String getMessageFromConfig(Player player, Key messageKey) {
        String message = messageKey.getConfigManager().getFileConfiguration().getString(messageKey.getPath(), null);
        if (message == null) {
            BlockHunt.plugin.getLogger().log(Level.SEVERE, "Unable to read value for " + messageKey.getPath());
            if (player != null) {
                player.sendMessage("An error occurred while generating a BlockHunt message. " +
                        "Please notify an administrator.");
            }
        }
        return message;
    }

}
